
# q-expansion of newform 5880.2.a.bf, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5880
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [0], [0], [2], [-6], [4], [4], [6], [0], [6], [-6], [-4], [-8], [14], [4], [2], [12], [-12], [-10], [8], [4], [-6], [14], [-2], [16], [8], [-2], [-6], [8], [20], [10], [4], [6], [8], [-6], [-4], [-16], [6], [0], [18], [-4], [-14], [-18], [0], [-12], [-16], [12], [2], [26], [4], [-10], [4], [18], [4], [-18], [0], [22], [-14], [20], [-18], [-4], [16], [30], [6], [28], [-30], [32], [-14], [-6], [4], [-32], [-26], [28], [-32], [-34], [-6], [10], [-34], [12], [38], [-20], [22], [32], [-16], [-30], [-6], [6], [24], [-4], [-8], [40], [-8], [-12], [-40], [-2], [-30], [28], [-2], [12], [-42], [12], [18], [20], [22], [-12], [10], [-28], [-26], [32], [6], [-38], [4], [32], [10], [28], [32], [-18], [-8], [-22], [-30], [-50], [-16], [-36], [-26], [22], [16], [48], [2], [-52], [-4], [-32], [22], [-22], [46], [-2], [52], [54], [-22], [-44], [6], [-48], [-24], [-6], [0], [-14], [42], [44], [-36], [-18], [-54], [-44], [-48], [28], [-36], [-40], [18], [14], [30], [32], [34], [-32], [-36], [18], [-24], [-32], [-46], [2], [14], [0], [2], [36], [10], [8], [-46], [36], [30], [-48], [-62], [24], [-12], [38], [-30], [-36], [46], [-2], [36], [-54], [0], [14], [24], [52], [-42], [48], [-46], [-30], [-34], [2], [-48], [-34], [8], [-22], [-26], [-36], [38], [40], [32], [-54], [-4], [34], [6], [8], [-52], [-64], [-10], [-32], [-6], [-44], [14], [-42], [24], [-30], [32], [-28], [-42], [-6], [-12], [48], [-24], [14], [-44], [16], [-6], [-28], [32], [54], [-18], [72], [24], [-72], [-28], [-40], [-50], [-14], [0], [24], [-52], [20], [-36], [-34], [74], [44], [-66], [54], [24], [-58], [-28], [-66], [62], [-40], [72], [-14], [10], [-38], [44], [62], [66], [-44], [-74], [-62], [20], [-10], [0], [46], [32], [-72], [-66], [-70], [-20], [-16], [-80], [-24], [42], [-68], [-36], [2], [-18], [-40], [-14], [30], [68], [-54], [20], [62], [70], [-24], [54], [52], [4], [46], [14], [24], [-16], [-12], [50], [0], [-22], [-60], [54], [0], [-2], [-22], [-44], [-12], [14], [36], [-12], [14], [-54], [52], [26], [-26], [64], [82], [38], [92], [-60], [52], [86], [-82], [-34], [0], [28], [-28], [-12], [-34], [82], [62], [8], [-26], [42], [-58], [8], [-66], [56], [66], [44], [16], [-2], [-12], [74], [-2], [-64], [-42], [-30], [32], [-20], [90], [-84], [86], [58], [-12], [-48], [-44], [10], [-34], [80], [-54], [-16], [44], [84], [22], [-40], [-66], [-92], [-44], [-42], [10], [54], [-42], [34], [-8], [74], [-4], [24], [0], [74], [84], [72], [-62], [6], [-16], [76], [-12], [74], [56], [50], [-28], [-18], [34], [42], [88], [-22], [-42], [0], [14], [2], [36], [60], [22], [22], [-64], [-28], [-86], [30], [20], [16], [42], [-48], [102], [98], [20], [92], [-2], [54], [-40], [18], [52], [-80]]
